﻿Param
<#(   $serviceName = "shashbanCloudService",
        $storageAccountName = "shashbancloudservice",
        $packageLocation = "C:\Windows\DtlDownloads\SampleCloudApp-AzureCloudService1_New-Component\AzureCloudService1.cspkg",
        $cloudConfigLocation = "C:\Windows\DtlDownloads\SampleCloudApp-AzureCloudService1_New-Component\ServiceConfiguration.Cloud.cscfg",
        $environment = "Production",
        $deploymentLabel = "ContinuousDeploy to $servicename",
        $timeStampFormat = "g",
        $alwaysDeleteExistingDeployments = 1,
        $enableDeploymentUpgrade = 1,
        $selectedsubscription = "Visual Studio Ultimate with MSDN",
        $subscriptionDataFile = "C:\Windows\DtlDownloads\SampleCloudApp-AzureCloudService1_New-Component\shashankb.publishsettings"
     )
#>    
	  (  $serviceName = $serName,
        $storageAccountName = $storName,
        $packageLocation = $applicationPath + $pkgLocation,
        $cloudConfigLocation = $applicationPath + $cscfgLocation,
        $environment = $env,
        $deploymentLabel = "ContinuousDeploy to $servicename",
        $timeStampFormat = "g",
        $alwaysDeleteExistingDeployments = 1,
        $enableDeploymentUpgrade = 1,
        $selectedsubscription = $sub,
        $subscriptionDataFile = $applicationPath + $subFile
     )

function Publish()
{
    $deployment = Get-AzureDeployment -ServiceName $serviceName -Slot $slot -ErrorVariable a -ErrorAction silentlycontinue 
    if ($a[0] -ne $null)
    {
        Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - No deployment is detected. Creating a new deployment. "
    }
    #check for existing deployment and then either upgrade, delete + deploy, or cancel according to $alwaysDeleteExistingDeployments and $enableDeploymentUpgrade boolean variables
    if ($deployment.Name -ne $null)
    {
        switch ($alwaysDeleteExistingDeployments)
        {
            1 
            {
                switch ($enableDeploymentUpgrade)
                {
                    1  #Update deployment inplace (usually faster, cheaper, won't destroy VIP)
                    {
                        Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Deployment exists in $servicename.  Upgrading deployment."
                        UpgradeDeployment
                    }
                    0  #Delete then create new deployment
                    {
                        Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Deployment exists in $servicename.  Deleting deployment."
                        DeleteDeployment
                        CreateNewDeployment
                        
                    }
                } # switch ($enableDeploymentUpgrade)
            }
            0
            {
                Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - ERROR: Deployment exists in $servicename.  Script execution cancelled."
                exit
            }
        } #switch ($alwaysDeleteExistingDeployments)
    } else {
            CreateNewDeployment
    }
}

function CreateNewDeployment()
{
    write-progress -id 3 -activity "Creating New Deployment" -Status "In progress"
    Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Creating New Deployment: In progress"

    $opstat = New-AzureDeployment -Slot $slot -Package $packageLocation -Configuration $cloudConfigLocation -label $deploymentLabel -ServiceName $serviceName
        
    $completeDeployment = Get-AzureDeployment -ServiceName $serviceName -Slot $slot
    $completeDeploymentID = $completeDeployment.deploymentid

    write-progress -id 3 -activity "Creating New Deployment" -completed -Status "Complete"
    Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Creating New Deployment: Complete, Deployment ID: $completeDeploymentID"
    
    StartInstances
}

function UpgradeDeployment()
{
    write-progress -id 3 -activity "Upgrading Deployment" -Status "In progress"
    Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Upgrading Deployment: In progress"
	Write-Verbose -Verbose "Set-AzureDeployment -Upgrade -Slot $slot -Package $packageLocation -Configuration $cloudConfigLocation -label $deploymentLabel -ServiceName $serviceName -Force"
    # perform Update-Deployment
    $setdeployment = Set-AzureDeployment -Upgrade -Slot $slot -Package $packageLocation -Configuration $cloudConfigLocation -label $deploymentLabel -ServiceName $serviceName -Force
    
    $completeDeployment = Get-AzureDeployment -ServiceName $serviceName -Slot $slot
    $completeDeploymentID = $completeDeployment.deploymentid
    
    write-progress -id 3 -activity "Upgrading Deployment" -completed -Status "Complete"
    Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Upgrading Deployment: Complete, Deployment ID: $completeDeploymentID"
}

function DeleteDeployment()
{

    write-progress -id 2 -activity "Deleting Deployment" -Status "In progress"
    Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Deleting Deployment: In progress"

    #WARNING - always deletes with force
    $removeDeployment = Remove-AzureDeployment -Slot $slot -ServiceName $serviceName -Force

    write-progress -id 2 -activity "Deleting Deployment: Complete" -completed -Status $removeDeployment
    Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Deleting Deployment: Complete"
    
}

function StartInstances()
{
    write-progress -id 4 -activity "Starting Instances" -status "In progress"
    Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Starting Instances: In progress"

    $deployment = Get-AzureDeployment -ServiceName $serviceName -Slot $slot
    $runstatus = $deployment.Status

    if ($runstatus -ne 'Running') 
    {
        $run = Set-AzureDeployment -Slot $slot -ServiceName $serviceName -Status Running
    }
    $deployment = Get-AzureDeployment -ServiceName $serviceName -Slot $slot
    $oldStatusStr = @("") * $deployment.RoleInstanceList.Count
    
    while (-not(AllInstancesRunning($deployment.RoleInstanceList)))
    {
        $i = 1
        foreach ($roleInstance in $deployment.RoleInstanceList)
        {
            $instanceName = $roleInstance.InstanceName
            $instanceStatus = $roleInstance.InstanceStatus

            if ($oldStatusStr[$i - 1] -ne $roleInstance.InstanceStatus)
            {
                $oldStatusStr[$i - 1] = $roleInstance.InstanceStatus
                Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Starting Instance '$instanceName': $instanceStatus"
            }

            write-progress -id (4 + $i) -activity "Starting Instance '$instanceName'" -status "$instanceStatus"
            $i = $i + 1
        }

        sleep -Seconds 1

        $deployment = Get-AzureDeployment -ServiceName $serviceName -Slot $slot
    }

    $i = 1
    foreach ($roleInstance in $deployment.RoleInstanceList)
    {
        $instanceName = $roleInstance.InstanceName
        $instanceStatus = $roleInstance.InstanceStatus

        if ($oldStatusStr[$i - 1] -ne $roleInstance.InstanceStatus)
        {
            $oldStatusStr[$i - 1] = $roleInstance.InstanceStatus
            Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Starting Instance '$instanceName': $instanceStatus"
        }

        $i = $i + 1
    }
    
    $deployment = Get-AzureDeployment -ServiceName $serviceName -Slot $slot
    $opstat = $deployment.Status 
    
    write-progress -id 4 -activity "Starting Instances" -completed -status $opstat
    Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Starting Instances: $opstat"
}

function AllInstancesRunning($roleInstanceList)
{
    foreach ($roleInstance in $roleInstanceList)
    {
        if ($roleInstance.InstanceStatus -ne "ReadyRole")
        {
            return $false
        }
    }
    
    return $true
}


		Write-Verbose -Verbose "serviceName = $serviceName"
        Write-Verbose -Verbose "storageAccountName = $storageAccountName"
        Write-Verbose -Verbose "packageLocation = $packageLocation" 
        Write-Verbose -Verbose "cloudConfigLocation = $cloudConfigLocation"
        Write-Verbose -Verbose "environment = $environment"
        Write-Verbose -Verbose "deploymentLabel = $deploymentLabel"
        Write-Verbose -Verbose "timeStampFormat = $timeStampFormat"
        Write-Verbose -Verbose "alwaysDeleteExistingDeployments = $alwaysDeleteExistingDeployments"
        Write-Verbose -Verbose "enableDeploymentUpgrade = $enableDeploymentUpgrade"
        Write-Verbose -Verbose "selectedsubscription = $selectedsubscription"
        Write-Verbose -Verbose "subscriptionDataFile = $subscriptionDataFile"

#configure powershell with Azure 1.7 modules
Import-Module Azure

#configure powershell with publishsettings for your subscription
Write-Verbose -Verbose "Import-AzurePublishSettingsFile $pubsettings"
$pubsettings = $subscriptionDataFile
Import-AzurePublishSettingsFile $pubsettings
Write-Verbose -Verbose "Set-AzureSubscription -CurrentStorageAccountName $storageAccountName -SubscriptionName $selectedsubscription"
Set-AzureSubscription -CurrentStorageAccountName $storageAccountName -SubscriptionName $selectedsubscription
Write-Verbose -Verbose "Select-AzureSubscription $selectedsubscription"
Select-AzureSubscription $selectedsubscription

#set remaining environment variables for Azure cmdlets
Write-Verbose -Verbose "Get-AzureSubscription $selectedsubscription"
$subscription = Get-AzureSubscription $selectedsubscription
$subscriptionname = $subscription.subscriptionname
$subscriptionid = $subscription.subscriptionid
$slot = $environment

#main driver - publish & write progress to activity log
Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Azure Cloud Service deploy script started."
Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Preparing deployment of $deploymentLabel for $subscriptionname with Subscription ID $subscriptionid."
Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Settings in use: packagelocation $packageLocation, environment $environment, servicename $serviceName, storageName $storageAccountName, configlocation $cloudconfiglocation"

Publish

Write-Verbose -Verbose "Get-AzureDeployment -slot $slot -serviceName $servicename"
$deployment = Get-AzureDeployment -slot $slot -serviceName $servicename
$deploymentUrl = $deployment.Url

Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Created Cloud Service with URL $deploymentUrl."
Write-Verbose -Verbose "$(Get-Date -f $timeStampFormat) - Azure Cloud Service deploy script finished."

